// Fill out your copyright notice in the Description page of Project Settings.

#include "ShoutBat.h"
#include "AudioActor.h"


// Sets default values
AAudioActor::AAudioActor()
{
	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

	VoiceCapture = FVoiceModule::Get().CreateVoiceCapture();
	if (VoiceCapture.IsValid())
	{
		VoiceCapture->Start();
	}

	VoiceCaptureAudioComponent = CreateDefaultSubobject<UAudioComponent>(TEXT("VoiceCaptureAudioComponent"));
	VoiceCaptureAudioComponent->SetupAttachment(RootComponent);
	VoiceCaptureAudioComponent->bAutoActivate = true;
	VoiceCaptureAudioComponent->bAlwaysPlay = true;
	//VoiceCaptureAudioComponent->PitchMultiplier = 0.85f;
	//VoiceCaptureAudioComponent->VolumeMultiplier = 5.f;
}

// Called when the game starts or when spawned
void AAudioActor::BeginPlay()
{
	Super::BeginPlay();

	if (AudioAtivo)
	{
		//GetWorldTimerManager().SetTimer(PlayVoiceCaptureTimer, this, &AAudioActor::PlayVoiceCapture, 0.4f, true, 0.f);

		if (VoiceCaptureSoundWaveProcedural == nullptr)
		{
			InitSoundProcedural();
		}
	}
}

// Called every frame
void AAudioActor::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);

	VoiceCaptureTick();

}


void AAudioActor::VoiceCaptureTick()
{
	if (!VoiceCapture.IsValid())
		return;

	if (!AudioAtivo)
		return;

	//UE_LOG(LogTemp, Warning, TEXT("VoiceCapture is valid"));

	// Get the Online Subsystem to work with
	IOnlineSubsystem*  OnlineSub = IOnlineSubsystem::Get();

	if (OnlineSub)
	{
		//UE_LOG(LogTemp, Warning, TEXT("SubSystem ok"));
		uint32 VoiceCaptureBytesAvailable = 0;
		EVoiceCaptureState::Type CaptureState = VoiceCapture->GetCaptureState(VoiceCaptureBytesAvailable);

		//UE_LOG(LogTemp, Warning, TEXT("Capture got"));

		VoiceCaptureBuffer.Reset();
		PlayVoiceCaptureFlag = false;

		if (CaptureState == EVoiceCaptureState::Ok && VoiceCaptureBytesAvailable > 0)
		{

			//UE_LOG(LogTemp, Warning, TEXT("Capture state ok"));

			int16_t VoiceCaptureSample;
			uint32 VoiceCaptureReadBytes;
			float VoiceCaptureTotalSquared = 0;

			VoiceCaptureBuffer.SetNumUninitialized(VoiceCaptureBytesAvailable);

			VoiceCapture->GetVoiceData(VoiceCaptureBuffer.GetData(), VoiceCaptureBytesAvailable, VoiceCaptureReadBytes);

			for (uint32 i = 0; i < (VoiceCaptureReadBytes / 2); i++)
			{
				VoiceCaptureSample = (VoiceCaptureBuffer[i * 2 + 1] << 8) | VoiceCaptureBuffer[i * 2];
				VoiceCaptureTotalSquared += ((float)VoiceCaptureSample * (float)VoiceCaptureSample);
			}

			float VoiceCaptureMeanSquare = (2 * (VoiceCaptureTotalSquared / VoiceCaptureBuffer.Num()));
			float VoiceCaptureRms = FMath::Sqrt(VoiceCaptureMeanSquare);
			float VoiceCaptureFinalVolume = ((VoiceCaptureRms / 32768.0) * 200.f);

			VoiceCaptureVolume = VoiceCaptureFinalVolume;

			//UE_LOG(LogTemp, Warning, TEXT("Start queue audio"));
			//UE_LOG(LogTemp, Warning, TEXT("Volume: %f"), VoiceCaptureVolume);
			//UE_LOG(LogTemp, Warning, TEXT("VoiceCaptureReadBytes: %d"), VoiceCaptureReadBytes);

			if (VoiceCaptureSoundWaveProcedural == nullptr)
			{
				InitSoundProcedural();
			}

			VoiceCaptureSoundWaveProcedural->QueueAudio(VoiceCaptureBuffer.GetData(), VoiceCaptureReadBytes);
			VoiceCaptureAudioComponent->SetSound(VoiceCaptureSoundWaveProcedural);


			PlayVoiceCaptureFlag = true;
		}

	}
}

void AAudioActor::InitSoundProcedural()
{
	VoiceCaptureSoundWaveProcedural = NewObject<USoundWaveProcedural>();
	VoiceCaptureSoundWaveProcedural->SampleRate = 16000;
	VoiceCaptureSoundWaveProcedural->NumChannels = 1;
	VoiceCaptureSoundWaveProcedural->Duration = INDEFINITELY_LOOPING_DURATION;
	VoiceCaptureSoundWaveProcedural->SoundGroup = SOUNDGROUP_Voice;
	VoiceCaptureSoundWaveProcedural->bLooping = false;
	VoiceCaptureSoundWaveProcedural->bProcedural = true;
	//VoiceCaptureSoundWaveProcedural->Pitch = 0.85f;
	//VoiceCaptureSoundWaveProcedural->Volume = 5.f;

	//UE_LOG(LogTemp, Warning, TEXT("Inicializando soundwave procedural"));
}

void AAudioActor::PlayVoiceCapture()
{
	if (!AudioAtivo)
		return;

	if (!PlayVoiceCaptureFlag)
	{
		VoiceCaptureAudioComponent->FadeOut(0.1f, 0.f);
		return;
	}

	if (VoiceCaptureAudioComponent->IsPlaying())
		return;

	VoiceCaptureAudioComponent->Play();
}